# coding: utf-8

"""
    Thoth User API

    No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)  # noqa: E501

    OpenAPI spec version: 0.6.0-dev

    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

from __future__ import absolute_import

import re  # noqa: F401

# python 2 and python 3 compatibility library
import six

from thamos.swagger_client.api_client import ApiClient


class PythonPackageMetadataApi(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    Ref: https://github.com/swagger-api/swagger-codegen
    """

    def __init__(self, api_client=None):
        if api_client is None:
            api_client = ApiClient()
        self.api_client = api_client

    def get_package_metadata(self, name, version, index, **kwargs):  # noqa: E501
        """Retrieve metadata relative to a Python Package from the Knowledge Graph.   # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_package_metadata(name, version, index, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str name: Name of the Python Package. (required)
        :param str version: Version of the Python Package. (required)
        :param str index: Index url of the Python Package. (required)
        :return: PythonPackageMetadataResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs["_return_http_data_only"] = True
        if kwargs.get("async_req"):
            return self.get_package_metadata_with_http_info(
                name, version, index, **kwargs
            )  # noqa: E501
        else:
            (data) = self.get_package_metadata_with_http_info(
                name, version, index, **kwargs
            )  # noqa: E501
            return data

    def get_package_metadata_with_http_info(
        self, name, version, index, **kwargs
    ):  # noqa: E501
        """Retrieve metadata relative to a Python Package from the Knowledge Graph.   # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_package_metadata_with_http_info(name, version, index, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str name: Name of the Python Package. (required)
        :param str version: Version of the Python Package. (required)
        :param str index: Index url of the Python Package. (required)
        :return: PythonPackageMetadataResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ["name", "version", "index"]  # noqa: E501
        all_params.append("async_req")
        all_params.append("_return_http_data_only")
        all_params.append("_preload_content")
        all_params.append("_request_timeout")

        params = locals()
        for key, val in six.iteritems(params["kwargs"]):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_package_metadata" % key
                )
            params[key] = val
        del params["kwargs"]
        # verify the required parameter 'name' is set
        if "name" not in params or params["name"] is None:
            raise ValueError(
                "Missing the required parameter `name` when calling `get_package_metadata`"
            )  # noqa: E501
        # verify the required parameter 'version' is set
        if "version" not in params or params["version"] is None:
            raise ValueError(
                "Missing the required parameter `version` when calling `get_package_metadata`"
            )  # noqa: E501
        # verify the required parameter 'index' is set
        if "index" not in params or params["index"] is None:
            raise ValueError(
                "Missing the required parameter `index` when calling `get_package_metadata`"
            )  # noqa: E501

        collection_formats = {}

        path_params = {}

        query_params = []
        if "name" in params:
            query_params.append(("name", params["name"]))  # noqa: E501
        if "version" in params:
            query_params.append(("version", params["version"]))  # noqa: E501
        if "index" in params:
            query_params.append(("index", params["index"]))  # noqa: E501

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params["Accept"] = self.api_client.select_header_accept(
            ["application/json"]
        )  # noqa: E501

        # Authentication setting
        auth_settings = []  # noqa: E501

        return self.api_client.call_api(
            "/python/package/metadata",
            "GET",
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type="PythonPackageMetadataResponse",  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get("async_req"),
            _return_http_data_only=params.get("_return_http_data_only"),
            _preload_content=params.get("_preload_content", True),
            _request_timeout=params.get("_request_timeout"),
            collection_formats=collection_formats,
        )



# coding: utf-8

"""
    Thoth User API

    No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)  # noqa: E501

    OpenAPI spec version: 0.6.0-dev

    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six


class BuildAnalysisResponse(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        "base_image_analysis": "BuildAnalysisResponseBaseImageAnalysis",
        "output_image_analysis": "BuildAnalysisResponseBaseImageAnalysis",
        "build_log_analysis": "BuildAnalysisResponseBaseImageAnalysis",
    }

    attribute_map = {
        "base_image_analysis": "base_image_analysis",
        "output_image_analysis": "output_image_analysis",
        "build_log_analysis": "build_log_analysis",
    }

    def __init__(
        self,
        base_image_analysis=None,
        output_image_analysis=None,
        build_log_analysis=None,
    ):  # noqa: E501
        """BuildAnalysisResponse - a model defined in Swagger"""  # noqa: E501
        self._base_image_analysis = None
        self._output_image_analysis = None
        self._build_log_analysis = None
        self.discriminator = None
        self.base_image_analysis = base_image_analysis
        self.output_image_analysis = output_image_analysis
        self.build_log_analysis = build_log_analysis

    @property
    def base_image_analysis(self):
        """Gets the base_image_analysis of this BuildAnalysisResponse.  # noqa: E501


        :return: The base_image_analysis of this BuildAnalysisResponse.  # noqa: E501
        :rtype: BuildAnalysisResponseBaseImageAnalysis
        """
        return self._base_image_analysis

    @base_image_analysis.setter
    def base_image_analysis(self, base_image_analysis):
        """Sets the base_image_analysis of this BuildAnalysisResponse.


        :param base_image_analysis: The base_image_analysis of this BuildAnalysisResponse.  # noqa: E501
        :type: BuildAnalysisResponseBaseImageAnalysis
        """
        if base_image_analysis is None:
            raise ValueError(
                "Invalid value for `base_image_analysis`, must not be `None`"
            )  # noqa: E501

        self._base_image_analysis = base_image_analysis

    @property
    def output_image_analysis(self):
        """Gets the output_image_analysis of this BuildAnalysisResponse.  # noqa: E501


        :return: The output_image_analysis of this BuildAnalysisResponse.  # noqa: E501
        :rtype: BuildAnalysisResponseBaseImageAnalysis
        """
        return self._output_image_analysis

    @output_image_analysis.setter
    def output_image_analysis(self, output_image_analysis):
        """Sets the output_image_analysis of this BuildAnalysisResponse.


        :param output_image_analysis: The output_image_analysis of this BuildAnalysisResponse.  # noqa: E501
        :type: BuildAnalysisResponseBaseImageAnalysis
        """
        if output_image_analysis is None:
            raise ValueError(
                "Invalid value for `output_image_analysis`, must not be `None`"
            )  # noqa: E501

        self._output_image_analysis = output_image_analysis

    @property
    def build_log_analysis(self):
        """Gets the build_log_analysis of this BuildAnalysisResponse.  # noqa: E501


        :return: The build_log_analysis of this BuildAnalysisResponse.  # noqa: E501
        :rtype: BuildAnalysisResponseBaseImageAnalysis
        """
        return self._build_log_analysis

    @build_log_analysis.setter
    def build_log_analysis(self, build_log_analysis):
        """Sets the build_log_analysis of this BuildAnalysisResponse.


        :param build_log_analysis: The build_log_analysis of this BuildAnalysisResponse.  # noqa: E501
        :type: BuildAnalysisResponseBaseImageAnalysis
        """
        if build_log_analysis is None:
            raise ValueError(
                "Invalid value for `build_log_analysis`, must not be `None`"
            )  # noqa: E501

        self._build_log_analysis = build_log_analysis

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(
                    map(lambda x: x.to_dict() if hasattr(x, "to_dict") else x, value)
                )
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(
                    map(
                        lambda item: (item[0], item[1].to_dict())
                        if hasattr(item[1], "to_dict")
                        else item,
                        value.items(),
                    )
                )
            else:
                result[attr] = value
        if issubclass(BuildAnalysisResponse, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, BuildAnalysisResponse):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



#!/usr/bin/env python3
# thamos
# Copyright(C) 2018, 2019, 2020 Fridolin Pokorny
#
# This program is free software: you can redistribute it and / or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

"""CLI and library configuration."""

import logging
import os
import typing
from urllib.parse import urljoin

import click
import requests
import yaml

from .utils import workdir
from .discover import discover_cpu
from .discover import discover_cuda_version
from .discover import discover_distribution
from .discover import discover_python_version
from .discover import discover_platform
from .exceptions import NoApiSupported
from .exceptions import NoRuntimeEnvironmentError
from .exceptions import ConfigurationError
from .exceptions import NoProjectDirError
from .exceptions import ServiceUnavailable

_LOGGER = logging.getLogger(__name__)
_THAMOS_DISABLE_TLS_WARNING = bool(int(os.getenv("THAMOS_DISABLE_TLS_WARNING", 0)))


class _Configuration:
    """Handling of Thoth's configuration."""

    DATA_DIR = os.path.join(os.path.dirname(os.path.realpath(__file__)), "data")
    DEFAULT_THOTH_CONFIG = os.path.join(DATA_DIR, "defaultThoth.yaml")
    CONFIG_NAME = ".thoth.yaml"
    REQUIREMENTS_FORMATS = frozenset(("pip", "pip-tools", "pip-compile", "pipenv"))
    _DEFAULT_REQUIREMENTS_FORMAT = "pipenv"
    _TLS_WARNING_LOGGED = False

    def __init__(self):
        """Construct configuration instance."""
        self._configuration = None
        self._index_aicoe = None
        # Thoth instance to be used when explicitly said by user - the one stated in
        # configuration file will be omitted.
        self.explicit_host = None
        self.tls_verify = None
        self._api_url = None

    @property
    def api_url(self):
        """Get URL to Thoth's API."""
        if not self._api_url:
            self._api_url = self.api_discovery(self.content["host"])

        return self._api_url

    @property
    def content(self):
        """Access configuration."""
        if not self._configuration:
            self.load_config()

        return self._configuration

    @property
    def requirements_format(self) -> str:
        """Check requirements_format in configuration."""
        requirements_format = self.content.get("requirements_format") or "pipenv"

        if not isinstance(requirements_format, str):
            raise ConfigurationError(
                "The data type for requirements_format should be str"
            )

        if requirements_format not in ("pip", "pip-tools", "pipenv"):
            raise ValueError(
                f"Unknown configuration option for requirements format: {requirements_format!r}"
            )

        return requirements_format

    def get_thoth_version(self) -> str:
        """Get version of Thoth backend."""
        _LOGGER.debug(
            "Contacting Thoth at %r to receive version information", self.api_url
        )
        response = requests.head(self.api_url, verify=self.tls_verify)
        response.raise_for_status()
        return response.headers.get("X-Thoth-Version", "Not Available")

    def config_file_exists(self) -> bool:
        """Check if configuration file exists."""
        try:
            with workdir(self.CONFIG_NAME):
                return True
        except NoProjectDirError:
            return False

    def load_config(self):
        """Load configuration file."""
        with workdir(config.CONFIG_NAME):
            with open(config.CONFIG_NAME, "r") as config_file:
                self._configuration = config_file.read()

                if int(os.getenv("THAMOS_CONFIG_EXPAND_ENV", 0)):
                    _LOGGER.info(
                        "Expanding configuration file based on environment variables"
                    )
                    self._configuration = self._configuration.format(**os.environ)

                self._configuration = yaml.safe_load(self._configuration)

    def create_default_config(
        self, template: str = None, nowrite: bool = False
    ) -> typing.Optional[dict]:
        """Place default configuration into the current directory."""
        if not os.path.isdir(".git"):
            _LOGGER.warning("Configuration file is not created in the root of git repo")

        template = template or self.DEFAULT_THOTH_CONFIG
        _LOGGER.debug("Reading configuration from %r", template)
        with open(template, "r") as default_config_file:
            default_config = default_config_file.read()

        _LOGGER.info("Discovering host runtime environment")

        cpu_info = discover_cpu()
        cuda_version = discover_cuda_version()
        # Add quotes for textual representation in the config file.
        cuda_version = f"'{cuda_version}" if cuda_version is not None else "null"
        os_name, os_version = discover_distribution()
        python_version = discover_python_version()
        platform = discover_platform()

        requirements_format = os.getenv(
            "THAMOS_REQUIREMENTS_FORMAT", self._DEFAULT_REQUIREMENTS_FORMAT
        )
        if requirements_format not in self.REQUIREMENTS_FORMATS:
            # This avoids possibly dangerous environment variable expansion.
            _LOGGER.warning(
                "Unknown requirements format specified, forcing %r: %r",
                self._DEFAULT_REQUIREMENTS_FORMAT,
                requirements_format,
            )
            requirements_format = self._DEFAULT_REQUIREMENTS_FORMAT

        expand_env = bool(int(os.getenv("THAMOS_CONFIG_EXPAND_ENV", 0)))
        default_config = default_config.format(
            cuda_version=cuda_version,
            os_name=os_name,
            os_version=os_version,
            python_version=python_version,
            platform=platform,
            requirements_format=requirements_format,
            **cpu_info,
            **(dict(os.environ) if expand_env else {}),
        )

        if not nowrite:
            _LOGGER.debug(
                "Writing configuration file to %r",
                os.path.join(os.getcwd(), self.CONFIG_NAME),
            )

            with open(self.CONFIG_NAME, "w") as config_file:
                config_file.write(default_config)
            return None
        else:
            return yaml.safe_load(default_config)

    @staticmethod
    def open_config_file():
        """Open Thoth's configuration file."""
        with workdir(config.CONFIG_NAME):
            _LOGGER.debug("Opening configuration file %r", config.CONFIG_NAME)
            click.edit(filename=config.CONFIG_NAME)

    def list_runtime_environments(self):
        """List available runtime environments."""
        return self.content.get("runtime_environments", [])

    def get_runtime_environment(self, name: str = None) -> typing.Optional[dict]:
        """Get runtime environment, retrieve the first runtime environment (the default one) if no name is provided."""
        content = self.content
        if "runtime_environments" not in content:
            raise NoRuntimeEnvironmentError(
                "No runtime environment configuration stated in the configuration file "
                "under 'runtime_environments' configuration entry"
            )

        if not isinstance(content["runtime_environments"], list):
            raise ConfigurationError(
                "The data type for requirements_format should be list"
            )

        to_return = None
        seen_names = set()
        for idx, runtime_environment in enumerate(content["runtime_environments"]):
            if not isinstance(runtime_environment, dict):
                raise ConfigurationError(
                    "Unknown runtime configuration entry, runtime configuration should be "
                    "a dictionary; got: %r",
                    runtime_environment,
                )

            # We explicitly iterate over all entries to perform the following sanity checks.
            current_name = runtime_environment.get("name")
            if current_name is not None and current_name in seen_names:
                raise ConfigurationError(
                    "Multiple configuration options with name %r found in the configuration file",
                    current_name,
                )

            if idx > 0 and current_name is None:
                raise ConfigurationError(
                    "Assign explicitly name for each configuration entry if there are multiple "
                    "runtime configuration options to distinguish between them"
                )

            if current_name is not None:
                seen_names.add(current_name)

            if name is None and idx == 0:
                # Return the first one by default.
                to_return = runtime_environment
            elif current_name == name:
                # Return by name.
                to_return = runtime_environment

        if to_return is None and len(content["runtime_environments"]) > 0:
            if name is not None:
                raise NoRuntimeEnvironmentError(
                    f"No runtime environment with name {name!r} was found in the configuration file; "
                    f"configured runtime environment names: {','.join(seen_names)}"
                )

            raise NoRuntimeEnvironmentError(
                "No runtime environment configuration was found"
            )

        return to_return

    def api_discovery(self, host: str) -> str:
        """Discover API versions available, return the most recent one supported by client and server."""
        api_url = urljoin("https://" + host, "api/v1")
        self.tls_verify = (
            self.tls_verify
            if self.tls_verify is not None
            else self.content.get("tls_verify", True)
        )

        if (
            not self.tls_verify
            and not _THAMOS_DISABLE_TLS_WARNING
            and not self._TLS_WARNING_LOGGED
        ):
            self._TLS_WARNING_LOGGED = True
            _LOGGER.warning(
                "TLS verification turned off, its highly recommended to use a secured connection, "
                "see configuration file for configuration options"
            )

        response = requests.get(
            api_url, verify=self.tls_verify, headers={"Accept": "application/json"}
        )

        try:
            response.raise_for_status()
        except Exception as exc:
            if response.status_code == 503:
                _LOGGER.error("Thoth service at %r is unavailable (HTTP 503)", api_url)
                raise ServiceUnavailable(str(exc))

            raise NoApiSupported(
                "Server does not support API v1 required by Thamos client"
            ) from exc

        self._api_url = api_url
        return self._api_url


config = _Configuration()
